
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'bin/tapper-mcp',
    'bin/tapper-mcp-daemon',
    'lib/Tapper/MCP.pm',
    'lib/Tapper/MCP/Child.pm',
    'lib/Tapper/MCP/Config.pm',
    'lib/Tapper/MCP/Control.pm',
    'lib/Tapper/MCP/Daemon.pm',
    'lib/Tapper/MCP/Info.pm',
    'lib/Tapper/MCP/Master.pm',
    'lib/Tapper/MCP/Net.pm',
    'lib/Tapper/MCP/Net/Reset/DoNothing.pm',
    'lib/Tapper/MCP/Net/Reset/Exec.pm',
    'lib/Tapper/MCP/Net/Reset/OSRC.pm',
    'lib/Tapper/MCP/Net/Reset/PM211MIP.pm',
    'lib/Tapper/MCP/Net/TAP.pm',
    'lib/Tapper/MCP/Plugin.pm',
    'lib/Tapper/MCP/Plugin/Test/All.pm',
    'lib/Tapper/MCP/Scheduler/Algorithm.pm',
    'lib/Tapper/MCP/Scheduler/Algorithm/DummyAlgorithm.pm',
    'lib/Tapper/MCP/Scheduler/Algorithm/WFQ.pm',
    'lib/Tapper/MCP/Scheduler/Builder.pm',
    'lib/Tapper/MCP/Scheduler/Controller.pm',
    'lib/Tapper/MCP/Scheduler/Host.pm',
    'lib/Tapper/MCP/Scheduler/Job.pm',
    'lib/Tapper/MCP/Scheduler/ObjectBuilder.pm',
    'lib/Tapper/MCP/Scheduler/PrioQueue.pm',
    'lib/Tapper/MCP/Scheduler/Queue.pm',
    'lib/Tapper/MCP/State.pm',
    'lib/Tapper/MCP/State/Details.pm',
    'lib/Tapper/MCP/State/Plugin/Dummy.pm',
    'lib/Tapper/MCP/State/Plugin/OSRC.pm',
    't/00-load.t',
    't/01-instance.t',
    't/author-eol.t',
    't/author-no-tabs.t',
    't/author-pod-syntax.t',
    't/bin/sleep.sh',
    't/command_files/install-error-msg.txt',
    't/command_files/install-error.txt',
    't/command_files/install-success.txt',
    't/command_files/quit_during_installation.txt',
    't/command_files/quit_during_test.txt',
    't/command_files/reboot_success.txt',
    't/command_files/test-error.txt',
    't/command_files/test-error2.txt',
    't/command_files/test-error_with_colon.txt',
    't/command_files/test-one-prc.txt',
    't/command_files/test-two-prc.txt',
    't/fixtures/testrundb/scenario_testruns.yml',
    't/fixtures/testrundb/testrun_double.yml',
    't/fixtures/testrundb/testrun_empty.yml',
    't/fixtures/testrundb/testrun_local.yml',
    't/fixtures/testrundb/testrun_with_autoinstall.yml',
    't/fixtures/testrundb/testrun_with_circle.yml',
    't/fixtures/testrundb/testrun_with_cobbler.yml',
    't/fixtures/testrundb/testrun_with_preconditions.yml',
    't/fixtures/testrundb/testrun_with_scheduling.yml',
    't/fixtures/testrundb/testrun_with_scheduling_deny.yml',
    't/fixtures/testrundb/testrun_with_scheduling_features.yml',
    't/fixtures/testrundb/testrun_with_scheduling_long.yml',
    't/fixtures/testrundb/testrun_with_scheduling_run1.yml',
    't/fixtures/testrundb/testrun_with_scheduling_run2.yml',
    't/fixtures/testrundb/testrun_with_scheduling_run4.yml',
    't/fixtures/testrundb/testrun_with_simnow.yml',
    't/fixtures/testrundb/testrun_with_xenpreconditions.yml',
    't/misc_files/bin/temare',
    't/misc_files/source_grub.lst',
    't/misc_files/suse/SLES10SP3_x86_64.lst',
    't/tapper-mcp-child-plugin.t',
    't/tapper-mcp-child-quit.t',
    't/tapper-mcp-child-wait.t',
    't/tapper-mcp-child.t',
    't/tapper-mcp-config-autoinstall.t',
    't/tapper-mcp-config-cobbler.t',
    't/tapper-mcp-config-producer.t',
    't/tapper-mcp-config-simnow.t',
    't/tapper-mcp-config.t',
    't/tapper-mcp-info.t',
    't/tapper-mcp-local.t',
    't/tapper-mcp-master-memory.t',
    't/tapper-mcp-master.t',
    't/tapper-mcp-net-tap.t',
    't/tapper-mcp-net.t',
    't/tapper-mcp-producer-kernel.t',
    't/tapper-mcp-producer-newest-package.t',
    't/tapper-mcp-producer-simnow-kernel.t',
    't/tapper-mcp-scheduler-algorithm-dummy.t',
    't/tapper-mcp-scheduler-algorithm-wfq.t',
    't/tapper-mcp-scheduler-denyhost.t',
    't/tapper-mcp-scheduler-interdep.t',
    't/tapper-mcp-scheduler-requested-features.t',
    't/tapper-mcp-scheduler-with-db-run1.t',
    't/tapper-mcp-scheduler-with-db-run2.t',
    't/tapper-mcp-scheduler-with-db-run3.t',
    't/tapper-mcp-scheduler-with-db-run4.t',
    't/tapper-mcp-ssh.t',
    't/tapper-mcp-state-keep-alive.t',
    't/tapper-mcp-state-prctimeout.t',
    't/tapper-mcp-state-revive.t',
    't/tapper-mcp-state-states.t',
    't/tapper-mcp-state-timeout.t',
    't/tapper-mcp-state-valid.t',
    't/tapper-mcp-state.t',
    't/tapper-mcp-testprogram-virt-guest.t'
);

notabs_ok($_) foreach @files;
done_testing;
